package mnk.replace.activities;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.support.annotation.NonNull;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.FragmentTransaction;
import android.view.View;
import android.support.design.widget.NavigationView;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;

import mnk.replace.R;
import mnk.replace.fragments.AddItemFragment;
import mnk.replace.fragments.DetailItemFragment;
import mnk.replace.fragments.DetailUserFragment;
import mnk.replace.fragments.ElementsListFragment;
import mnk.replace.fragments.ElementsMapFragment;
import mnk.replace.fragments.FilterFragment;
import mnk.replace.fragments.LoginFragment;
import mnk.replace.fragments.SettingsFragment;
import mnk.replace.manager.CurrentLocationManager;
import mnk.replace.manager.FirebaseManager;
import mnk.replace.model.ItemContent;
import mnk.replace.model.UserContent;

public class NavigationDrawerActivity extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener, GoogleApiClient.OnConnectionFailedListener {

    // SERVER DATA
    public FirebaseManager firebaseManager;

    // LOCATION
    public CurrentLocationManager currentLocationManager;

    // FRAGMENTS
    public FragmentTransaction fragmentsTransaction;
    public AddItemFragment addItemFragment;
    public DetailItemFragment detailItemFragment;
    public DetailUserFragment detailUserFragment;
    public ElementsListFragment elementsListFragment;
    public ElementsMapFragment elementsMapFragment;
    public FilterFragment filterFragment;
    public LoginFragment loginFragment;
    public SettingsFragment settingsFragment;

    // UI
    NavigationView navigationView;
    public FloatingActionButton fab;
    boolean showList = true;
    public MenuItem deleteItem;
    public MenuItem filterItem;
    public MenuItem viewItemsSelector;
    public DrawerLayout drawer;
    View headerview;
    TextView textViewProfileName;
    TextView textViewProfileEmail;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // INIT FRAGMENTS
        addItemFragment = new AddItemFragment();
        detailItemFragment = new DetailItemFragment();
        detailUserFragment = new DetailUserFragment();
        elementsListFragment = new ElementsListFragment();
        elementsMapFragment = new ElementsMapFragment();
        filterFragment = new FilterFragment();
        loginFragment = new LoginFragment();
        settingsFragment = new SettingsFragment();

        // INIT LOCATION
        currentLocationManager = new CurrentLocationManager(this);

        // INIT SERVER DATA
        firebaseManager = new FirebaseManager(this);

        // TOOLBAR
        setContentView(R.layout.activity_navigation_drawer);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // FLOATING BUTTON
        fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View view) {
                addItemFragment = new AddItemFragment();
                fragmentsTransaction = getSupportFragmentManager().beginTransaction();
                fragmentsTransaction.replace(R.id.frame, addItemFragment).addToBackStack(null).commit();
            }

        });

        // DRAWER LAYOUT
        drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close) {
            @Override
            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                updateUIUser();
            }
        };
        drawer.addDrawerListener(toggle);
        toggle.syncState();

        navigationView = (NavigationView) findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);

        // HEADER
        headerview = navigationView.getHeaderView(0);
        textViewProfileName = (TextView) headerview.findViewById(R.id.nav_header_textView1);
        textViewProfileEmail = (TextView) headerview.findViewById(R.id.nav_header_textView2);
        headerview.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
            fragmentsTransaction = getSupportFragmentManager().beginTransaction();
            if(firebaseManager.getFirebaseUser() == null) {
                loginFragment = new LoginFragment();
                fragmentsTransaction.replace(R.id.frame, loginFragment);
            }
            else {
                detailUserFragment = new DetailUserFragment();
                fragmentsTransaction.replace(R.id.frame, detailUserFragment);
            }
            fragmentsTransaction.addToBackStack(null).commit();
            DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
            drawer.closeDrawer(GravityCompat.START);
            }

        });

    }

    // GOOGLE RETURN ACTIVITY
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        firebaseManager.onActivityResult(requestCode,resultCode,data);
    }

    // LOCATION RETURN ACTIVITY
    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           @NonNull String permissions[],
                                           @NonNull int[] grantResults) {
        currentLocationManager.onRequestPermissionsResult(requestCode, permissions, grantResults);

    }

    // SHOW PERMISSION DIALOG
    public void showGPSPermissionDialog() {
        DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which){
                    case DialogInterface.BUTTON_NEUTRAL:
                        currentLocationManager.requestPermissions();
                        break;
                }
            }
        };
        AlertDialog.Builder builder = new AlertDialog.Builder(NavigationDrawerActivity.this);
        builder.setMessage("This app needs access to GPS. Please allow it in next dialog.").setNeutralButton("Ok", dialogClickListener).show();
    }

    // EVENT FIRED ON LOCATION CHANGED
    public void refreshLocation() {
        elementsMapFragment.updateLocationUI();
    }

    // LAUNCH MAP ACTIVITY
    public void launchLocationActivity() {
        // LAUNCH MAP'S FRAGMENT
        fragmentsTransaction = getSupportFragmentManager().beginTransaction();
        fragmentsTransaction.replace(R.id.frame, elementsMapFragment).commit();
    }

    // UI NAVIGATION DRAWER HEADER UPDATE
    public void updateUIUser() {
        if(firebaseManager.getFirebaseUser() == null) {
            textViewProfileName.setText("Please Register");
            textViewProfileEmail.setText("To Access All Features");
        }
        else {
            textViewProfileName.setText(firebaseManager.getCurrentUserDisplayName());
            textViewProfileEmail.setText(firebaseManager.getCurrentUserEmail());
        }
    }

    // LAUNCH DETAIL ITEM FRAGMENT
    public void launchDetailItemFragmentById(String id) {
        fragmentsTransaction = getSupportFragmentManager().beginTransaction();
        fragmentsTransaction.replace(R.id.frame, detailItemFragment).addToBackStack(null).commit();
        detailItemFragment.Item = new ItemContent.Item(firebaseManager.itemContent.getItemById(id));
    }

    @Override
    public void onBackPressed() {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) { drawer.closeDrawer(GravityCompat.START); }
        else {

            if(detailItemFragment.isImageFitToScreen) { detailItemFragment.setFullscreenImage(); }
            else { super.onBackPressed(); }

        }

        // FAB VISIBILITY
        fab.setVisibility(View.VISIBLE);

        // ACTION BAR VISIBILITY
        actionBarShowDeleteItem(false);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.navigation_drawer, menu);
        deleteItem = menu.findItem(R.id.action_delete_item);
        filterItem = menu.findItem(R.id.action_settings);
        viewItemsSelector = menu.findItem(R.id.action_show_list);
        return true;
    }

    public void actionBarShowDeleteItem(boolean inValue) {
        deleteItem.setVisible(inValue);
    }

    public void actionBarShowFilterItem(boolean inValue) {
        filterItem.setVisible(inValue);
    }

    public void actionBarShowViewItemsSelector(boolean visible, boolean type) {
        viewItemsSelector.setVisible(visible);
        if(type) { viewItemsSelector.setIcon(android.R.drawable.ic_dialog_map); }
        else { viewItemsSelector.setIcon(android.R.drawable.ic_menu_sort_by_size); }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.action_settings) {
            fragmentsTransaction = getSupportFragmentManager().beginTransaction();
            fragmentsTransaction.replace(R.id.frame, filterFragment);
            fragmentsTransaction.addToBackStack(null).commit();
        }
        else if (id == R.id.action_show_list) {
            fragmentsTransaction = getSupportFragmentManager().beginTransaction();
            if (showList == true) {
                showList = false;
                fragmentsTransaction.replace(R.id.frame, elementsListFragment).addToBackStack(null).commit();
                item.setIcon(android.R.drawable.ic_dialog_map);
            }
            else {
                showList = true;
                fragmentsTransaction.replace(R.id.frame, elementsMapFragment).addToBackStack(null).commit();
                item.setIcon(android.R.drawable.ic_menu_sort_by_size);
            }
        }
        return super.onOptionsItemSelected(item);
    }

    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.nav_manage) {
            fragmentsTransaction = getSupportFragmentManager().beginTransaction();
            fragmentsTransaction.replace(R.id.frame, settingsFragment);
            fragmentsTransaction.addToBackStack(null).commit();
            DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
            drawer.closeDrawer(GravityCompat.START);
        }
        return true;
    }

    @Override
    public void onConnectionFailed(@NonNull ConnectionResult connectionResult) { }

}
