package mnk.replace.fragments;

import android.app.Activity;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.v4.app.Fragment;
import android.support.v4.content.FileProvider;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.UploadTask;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Random;

import mnk.replace.activities.NavigationDrawerActivity;
import mnk.replace.R;
import mnk.replace.model.ItemContent;

import static android.app.Activity.RESULT_OK;

/**
 * A simple {@link Fragment} subclass.
 */
public class AddItemFragment extends Fragment {

    NavigationDrawerActivity parentActivity;

    // UI
    ImageView imageView_uploaditem;
    TextView editText_description;
    Button button_uploaditem;
    ProgressBar progressBar_upload;
    Thread progressBarThread;
    View view;

    // PROGRESS BAR
    int pStatus = 0;
    Handler handler = new Handler();

    // CAMERA
    private static final int CAMERA_PIC_REQUEST = 1337;
    boolean takenPhoto;

    // LOCAL STORAGE
    private StorageReference mStorageRef;

    // RANDOM ID
    String randomID;

    // DATE
    String dateTime;

    public AddItemFragment() {
        // Required empty public constructor
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        view = inflater.inflate(R.layout.fragment_add_item, container, false);
        parentActivity = ((NavigationDrawerActivity)getActivity());
        imageView_uploaditem = (ImageView) view.findViewById(R.id.imageView_uploaditem);
        editText_description = (TextView) view.findViewById(R.id.editText_description);
        editText_description.setText("");
        button_uploaditem = (Button) view.findViewById(R.id.button_uploaditem);
        progressBar_upload = (ProgressBar) view.findViewById(R.id.progressBar_upload);
        progressBar_upload.setVisibility(View.INVISIBLE);
        takenPhoto = false;
        parentActivity.fab.setVisibility(View.INVISIBLE);

        imageView_uploaditem.setOnClickListener(new View.OnClickListener() {
            //@Override
            public void onClick(View v) {
                // START CAMERA APP
                Intent intent = new Intent("android.media.action.IMAGE_CAPTURE");
                startActivityForResult(intent, CAMERA_PIC_REQUEST);
            }
        });

        button_uploaditem.setOnClickListener(new View.OnClickListener() {
            //@Override
            public void onClick(View v) {

            if((editText_description.getText().length() == 0) && (takenPhoto == false)) {
                Toast.makeText(getContext(), "Empty item not allowed", Toast.LENGTH_SHORT).show();
                return;
            }

            randomID = createrRandomId();
            dateTime = getDateTime();

            if(takenPhoto == true) {

                // CONSTRUCT IMAGE
                Bitmap bitmap = ((BitmapDrawable) imageView_uploaditem.getDrawable()).getBitmap();

                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                bitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos);
                byte[] bitmapData = baos.toByteArray();

                // START PROGRESS BAR ANIMATE
                progressBarAnimate(true);

                // UPLOAD IMAGE
                mStorageRef = FirebaseStorage.getInstance().getReference("images/" + randomID + ".jpg");
                UploadTask uploadTask = mStorageRef.putBytes(bitmapData);

                uploadTask.addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {

                        // SHOW MESSAGE ERROR
                        Toast.makeText(getContext(), "Failed", Toast.LENGTH_LONG).show();
                        // STOP PROGRESS BAR ANIMATE
                        progressBarAnimate(false);

                    }
                });

                uploadTask.addOnSuccessListener(new OnSuccessListener<UploadTask.TaskSnapshot>() {

                    @Override
                    public void onSuccess(UploadTask.TaskSnapshot taskSnapshot) {

                    // CONSTRUCT NEW ITEM
                    ItemContent.Item tempItem = new ItemContent.Item();
                    tempItem.setDescription(editText_description.getText().toString());
                    tempItem.setLatitude(parentActivity.currentLocationManager.mLastKnownLocation.getLatitude());
                    tempItem.setLongitude(parentActivity.currentLocationManager.mLastKnownLocation.getLongitude());

                    if (parentActivity.firebaseManager.getFirebaseUser() != null) {
                        tempItem.setUserId(parentActivity.firebaseManager.getFirebaseUser().getUid());
                    } else {
                        tempItem.setUserId("Anonymous");
                    }

                    tempItem.setImageURL(taskSnapshot.getDownloadUrl().toString());
                    tempItem.setId(randomID);
                    tempItem.setDateTime(dateTime);

                    // FIREBASE INIT & UPLOAD NEW ITEM
                    DatabaseReference usersRef = parentActivity.firebaseManager.getFirebaseDataReference().child("items/" + randomID);
                    usersRef.setValue(tempItem);

                    // STOP PROGRESS BAR ANIMATE
                    progressBarAnimate(false);

                    // SHOW MESSAGE DONE
                    Toast.makeText(getContext(), "Uploaded correctly", Toast.LENGTH_LONG).show();

                    // FAB VISIBILITY
                    parentActivity.fab.setVisibility(View.VISIBLE);

                    // POP BACK
                    getActivity().getSupportFragmentManager().popBackStackImmediate();

                    }

                });

            }
            else {

                // START PROGRESS BAR ANIMATE
                progressBarAnimate(true);

                // CONSTRUCT NEW ITEM
                ItemContent.Item tempItem = new ItemContent.Item();
                tempItem.setDescription(editText_description.getText().toString());
                tempItem.setLatitude(parentActivity.currentLocationManager.mLastKnownLocation.getLatitude());
                tempItem.setLongitude(parentActivity.currentLocationManager.mLastKnownLocation.getLongitude());

                if (parentActivity.firebaseManager.getFirebaseUser() != null) {
                    tempItem.setUserId(parentActivity.firebaseManager.getFirebaseUser().getUid());
                } else {
                    tempItem.setUserId("Anonymous");
                }

                tempItem.setDateTime(dateTime);
                tempItem.setId(randomID);

                // FIREBASE INIT & UPLOAD NEW ITEM
                DatabaseReference usersRef = parentActivity.firebaseManager.getFirebaseDataReference().child("items/" + randomID);
                usersRef.setValue(tempItem);

                // STOP PROGRESS BAR ANIMATE
                progressBarAnimate(false);

                // SHOW MESSAGE DONE
                Toast.makeText(getContext(), "Uploaded correctly", Toast.LENGTH_LONG).show();

                // FAB VISIBILITY
                parentActivity.fab.setVisibility(View.VISIBLE);

                // POP BACK
                getActivity().getSupportFragmentManager().popBackStackImmediate();

            }

            // Hide keyboard
            final InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(getView().getWindowToken(), 0);

            }

        });

        // ACTION BAR BUTTONS
        if(parentActivity.filterItem != null) {
            parentActivity.actionBarShowFilterItem(false);
            parentActivity.actionBarShowViewItemsSelector(false, false);
        }

        return view;
    }
    public String createrRandomId() {
        int MAX_LENGTH = 20;
        Random generator = new Random();
        StringBuilder randomStringBuilder = new StringBuilder();
        int randomLength = generator.nextInt(MAX_LENGTH);
        char tempChar;
        for (int i = 0; i < randomLength; i++) {
            tempChar = (char) (generator.nextInt(25) + 97);
            randomStringBuilder.append(tempChar);
        }

        if(parentActivity.firebaseManager.itemContent.checkUsedId(randomStringBuilder.toString())) {
            return createrRandomId();
        }
        else {
            return randomStringBuilder.toString();
        }

    }
    public String getDateTime() {

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault());
        Date date = new Date();
        return dateFormat.format(date);

    }
    public void progressBarAnimate (final boolean animateIn) {
        if(animateIn) {
            progressBar_upload.setVisibility(View.VISIBLE);
            progressBarThread = new Thread() {
                @Override
                public void run() {
                    while (pStatus <= 100) {
                        handler.post(new Runnable() {
                            @Override
                            public void run() {
                                progressBar_upload.setProgress(pStatus);
                                //txtProgress.setText(pStatus + " %");
                            }
                        });
                        try {
                            Thread.sleep(100);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                        pStatus++;
                    }
                }
            };
            progressBarThread.start();
        }
        else {
            progressBar_upload.setVisibility(View.INVISIBLE);
            progressBarThread.interrupt();
        }
    }
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == CAMERA_PIC_REQUEST) {
            if((data == null) || (data.getExtras() == null)) {
                takenPhoto = false;
            }
            else {
                // LOAD IMAGE IN IMAGEVIEW
                Bitmap image = (Bitmap) data.getExtras().get("data");
                imageView_uploaditem.setImageBitmap(image);
                takenPhoto = true;
            }
        }
    }
}
