package mnk.replace.fragments;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.auth.api.Auth;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.common.api.Status;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.auth.UserProfileChangeRequest;
import com.google.firebase.storage.FileDownloadTask;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;

import java.io.File;
import java.io.IOException;
import java.util.List;

import mnk.replace.activities.NavigationDrawerActivity;
import mnk.replace.R;
import mnk.replace.model.ItemContent;
import mnk.replace.model.UserContent;

/**
 * A simple {@link Fragment} subclass.
 */
public class DetailUserFragment extends Fragment {

    TextView editText_username;
    TextView editText_email;
    Button button_save;
    Button button_logout;
    NavigationDrawerActivity parentActivity;

    SimpleItemRecyclerViewAdapter adapter;

    public ItemContent localItemContent;

    public DetailUserFragment() {
        // Required empty public constructor
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_detail_user, container, false);
        parentActivity = ((NavigationDrawerActivity)getActivity());
        editText_username = (TextView) view.findViewById(R.id.editText_username);
        editText_email = (TextView) view.findViewById(R.id.editText_email);
        button_save = (Button) view.findViewById(R.id.button_save);
        button_logout = (Button) view.findViewById(R.id.button_logout);
        parentActivity.fab.setVisibility(View.INVISIBLE);

        if(parentActivity.firebaseManager.getFirebaseUser() != null) {
            UserContent.User cUser = parentActivity.firebaseManager.userContent.getUserById(parentActivity.firebaseManager.getCurrentUserUid());
            editText_username.setText(cUser.getNickName());
            editText_email.setText(cUser.getEmail());
        }
        button_save.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (parentActivity.firebaseManager.getFirebaseUser() != null) {
                    UserProfileChangeRequest profileUpdatesDisplayName = new UserProfileChangeRequest.Builder().setDisplayName(editText_username.getText().toString()).build();
                    parentActivity.firebaseManager.getFirebaseUser().updateProfile(profileUpdatesDisplayName);
                    parentActivity.firebaseManager.getFirebaseUser().updateEmail(editText_email.getText().toString());

                    UserContent.User updateUser = new UserContent.User();
                    updateUser.setId(parentActivity.firebaseManager.getCurrentUserUid());
                    updateUser.setNickName(editText_username.getText().toString());
                    updateUser.setEmail(editText_email.getText().toString());
                    parentActivity.firebaseManager.updateUserToDatabase(updateUser);

                    // Set Button Visibility
                    parentActivity.fab.setVisibility(View.VISIBLE);

                    // Hide keyboard
                    final InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(getView().getWindowToken(), 0);

                    // Show info
                    Toast.makeText(getContext(), "Updated", Toast.LENGTH_LONG).show();

                    getFragmentManager().popBackStack();
                }
            }
        });
        button_logout.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                logoutGoogle(parentActivity.firebaseManager.mGoogleApiClient);
                parentActivity.firebaseManager.getFirebaseAuth().signOut();
                parentActivity.firebaseManager.setFirebaseUser(null);
                parentActivity.updateUIUser();
                parentActivity.fab.setVisibility(View.VISIBLE);

                // Hide keyboard
                final InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.hideSoftInputFromWindow(getView().getWindowToken(), 0);

                // Show info
                Toast.makeText(getContext(), "Logged Out", Toast.LENGTH_LONG).show();

                getFragmentManager().popBackStack();
            }
        });


        // Init the RecyclerView and his adapter
        RecyclerView recyclerView = (RecyclerView) view.findViewById(R.id.recyclerView_items);
        adapter = new SimpleItemRecyclerViewAdapter();
        recyclerView.setAdapter(adapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));

        localItemContent = new ItemContent();
        localItemContent.removeItems();

        String userId = parentActivity.firebaseManager.getFirebaseUser().getUid();

        for(int x = 0; x < parentActivity.firebaseManager.itemContent.getItems().size(); x++) {
            //if(parentActivity.applyFilter(parentActivity.itemContent.getItem(x))) {

            if(parentActivity.firebaseManager.itemContent.getItem(x).getUserId().equals(userId)) {
                localItemContent.addItem(parentActivity.firebaseManager.itemContent.getItem(x));
            }
        }

        // ACTION BAR BUTTONS
        if(parentActivity.filterItem != null) {
            parentActivity.actionBarShowFilterItem(false);
            parentActivity.actionBarShowViewItemsSelector(false, false);
        }

        // Inflate the layout for this fragment
        return view;
    }

    public void refreshItems() {

        localItemContent = new ItemContent();
        localItemContent.removeItems();

        List<ItemContent.Item> filteredItems = parentActivity.firebaseManager.itemContent.getFilteredItems(parentActivity.currentLocationManager.mLastKnownLocation, parentActivity.filterFragment.filterDistance, parentActivity.filterFragment.filterTags);

        for(int x = 0; x < filteredItems.size(); x++) {
            localItemContent.addItem(filteredItems.get(x));
        }

        if(parentActivity.currentLocationManager.mLastKnownLocation != null) {
            localItemContent.sortByDistance(parentActivity.currentLocationManager.mLastKnownLocation);
        }

        adapter.notifyDataSetChanged();

    }

    public class SimpleItemRecyclerViewAdapter extends RecyclerView.Adapter<DetailUserFragment.SimpleItemRecyclerViewAdapter.ViewHolder>{

        public class ViewHolder extends RecyclerView.ViewHolder {

            TextView textView_cardView_description;
            TextView textView_cardView_distance;
            TextView textView_cardView_user;
            ImageView imageView_cardView_photo;
            int position;
            View mView;

            public ViewHolder(View itemView) {

                super(itemView);
                mView = itemView;
                textView_cardView_description = itemView.findViewById(R.id.textView_cardView_description);
                textView_cardView_distance = itemView.findViewById(R.id.textView_cardView_distance);
                textView_cardView_user = itemView.findViewById(R.id.textView_cardView_user);
                imageView_cardView_photo = itemView.findViewById(R.id.imageView_cardView_photo);

            }
        }

        @Override
        public DetailUserFragment.SimpleItemRecyclerViewAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {

            // Set the layout depending of the position of every element in the list
            View view = null;
            DetailUserFragment.SimpleItemRecyclerViewAdapter.ViewHolder viewHolder = null;
            LayoutInflater inflater = LayoutInflater.from(parent.getContext());
            view = inflater.inflate(R.layout.fragment_cardview_elements_list, parent, false);
            return new SimpleItemRecyclerViewAdapter.ViewHolder(view);

        }

        @Override
        public void onBindViewHolder(final DetailUserFragment.SimpleItemRecyclerViewAdapter.ViewHolder holder, int position) {

            if(localItemContent.getItem(position).getImageURL().equals("") == false) {

                FirebaseStorage storage = FirebaseStorage.getInstance();
                StorageReference storageRef = storage.getReferenceFromUrl(localItemContent.getItems().get(position).getImageURL());

                final File localFile;
                try {
                    localFile = File.createTempFile("images", "jpg");
                    storageRef.getFile(localFile).addOnSuccessListener(new OnSuccessListener<FileDownloadTask.TaskSnapshot>() {
                        @Override
                        public void onSuccess(FileDownloadTask.TaskSnapshot taskSnapshot) {
                            Bitmap bitmap = BitmapFactory.decodeFile(localFile.getAbsolutePath());
                            holder.imageView_cardView_photo.setImageBitmap(bitmap);

                        }
                    }).addOnFailureListener(new OnFailureListener() {
                        @Override
                        public void onFailure(@NonNull Exception exception) {
                        }
                    });
                } catch (IOException e) {
                    e.printStackTrace();
                }

            }

            holder.textView_cardView_description.setText(localItemContent.getItems().get(position).getDescription());
            holder.textView_cardView_distance.setText(Double.toString(localItemContent.getItem(position).getDistanceTo(parentActivity.currentLocationManager.mLastKnownLocation) / 1000.0) + " Km");
            UserContent.User tempUser = parentActivity.firebaseManager.userContent.getUserById(localItemContent.getItem(position).getUserId());
            if(tempUser == null) { holder.textView_cardView_user.setText("Anonymous"); }
            else { holder.textView_cardView_user.setText(tempUser.getNickName()); }

            holder.position = position;

            holder.mView.setOnClickListener(new View.OnClickListener() {

                // On click on item --> Start the detail in the same screen or start the detail activity
                @Override
                public void onClick(View v) {

                    int position = holder.position;
                    parentActivity.launchDetailItemFragmentById(localItemContent.getItems().get(position).getId());

                }

            });

        }

        @Override
        public int getItemCount() {
            return localItemContent.getItems().size();
        }

    }

    // LOGOUT GOOGLE
    public void logoutGoogle(GoogleApiClient inGoogleApiClient) {
        if(inGoogleApiClient != null)
            Auth.GoogleSignInApi.signOut(inGoogleApiClient).setResultCallback(new ResultCallback<Status>() {
                @Override
                public void onResult(@NonNull Status status) {
                    Toast.makeText(parentActivity.getBaseContext(), "Logged Out", Toast.LENGTH_LONG).show();
                }
            });
    }


}
