package mnk.replace.fragments;


import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.storage.FileDownloadTask;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;

import java.io.File;
import java.io.IOException;
import java.util.List;

import mnk.replace.activities.NavigationDrawerActivity;
import mnk.replace.R;
import mnk.replace.model.ItemContent;
import mnk.replace.model.UserContent;

/**
 * A simple {@link Fragment} subclass.
 */
public class ElementsListFragment extends Fragment {

    NavigationDrawerActivity parentActivity;
    public ItemContent localItemContent;
    SimpleItemRecyclerViewAdapter adapter;

    public ElementsListFragment() {
        // Required empty public constructor
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.fragment_elements_list, container, false);
        parentActivity = ((NavigationDrawerActivity)getActivity());

        // Init the RecyclerView and his adapter
        RecyclerView recyclerView = (RecyclerView) view.findViewById(R.id.fragment_elements_list);
        adapter = new SimpleItemRecyclerViewAdapter();
        recyclerView.setAdapter(adapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));

        localItemContent = new ItemContent();

        List<ItemContent.Item> filteredItems = parentActivity.firebaseManager.itemContent.getFilteredItems(parentActivity.currentLocationManager.mLastKnownLocation, parentActivity.filterFragment.filterDistance, parentActivity.filterFragment.filterTags);

        for(int x = 0; x < filteredItems.size(); x++) {
            localItemContent.addItem(filteredItems.get(x));
        }

        if(parentActivity.currentLocationManager.mLastKnownLocation != null) {
            localItemContent.sortByDistance(parentActivity.currentLocationManager.mLastKnownLocation);
        }

        // ACTION BAR BUTTONS
        if(parentActivity.filterItem != null) {
            parentActivity.actionBarShowFilterItem(true);
            parentActivity.actionBarShowViewItemsSelector(true, true);
        }

        return view;
    }

    public void refreshItems() {

        localItemContent = new ItemContent();
        localItemContent.removeItems();

        List<ItemContent.Item> filteredItems = parentActivity.firebaseManager.itemContent.getFilteredItems(parentActivity.currentLocationManager.mLastKnownLocation, parentActivity.filterFragment.filterDistance, parentActivity.filterFragment.filterTags);

        for(int x = 0; x < filteredItems.size(); x++) {
            localItemContent.addItem(filteredItems.get(x));
        }

        if(parentActivity.currentLocationManager.mLastKnownLocation != null) {
            localItemContent.sortByDistance(parentActivity.currentLocationManager.mLastKnownLocation);
        }

        adapter.notifyDataSetChanged();

    }

    public class SimpleItemRecyclerViewAdapter extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder>{

        public class ViewHolder extends RecyclerView.ViewHolder {

            TextView textView_cardView_description;
            TextView textView_cardView_distance;
            TextView textView_cardView_user;
            ImageView imageView_cardView_photo;
            int position;
            View mView;

            public ViewHolder(View itemView) {

                super(itemView);
                mView = itemView;
                textView_cardView_description = itemView.findViewById(R.id.textView_cardView_description);
                textView_cardView_distance = itemView.findViewById(R.id.textView_cardView_distance);
                textView_cardView_user = itemView.findViewById(R.id.textView_cardView_user);
                imageView_cardView_photo = itemView.findViewById(R.id.imageView_cardView_photo);

            }
        }

        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {

            // Set the layout depending of the position of every element in the list
            View view = null;
            SimpleItemRecyclerViewAdapter.ViewHolder viewHolder = null;
            LayoutInflater inflater = LayoutInflater.from(parent.getContext());
            view = inflater.inflate(R.layout.fragment_cardview_elements_list, parent, false);
            return new ViewHolder(view);

        }

        @Override
        public void onBindViewHolder(final ViewHolder holder, int position) {

            if(localItemContent.getItem(position).getImageURL().equals("") == false) {

                FirebaseStorage storage = FirebaseStorage.getInstance();
                StorageReference storageRef = storage.getReferenceFromUrl(localItemContent.getItem(position).getImageURL());

                final File localFile;
                try {
                    localFile = File.createTempFile("images", "jpg");
                    storageRef.getFile(localFile).addOnSuccessListener(new OnSuccessListener<FileDownloadTask.TaskSnapshot>() {
                        @Override
                        public void onSuccess(FileDownloadTask.TaskSnapshot taskSnapshot) {
                            Bitmap bitmap = BitmapFactory.decodeFile(localFile.getAbsolutePath());
                            holder.imageView_cardView_photo.setImageBitmap(bitmap);

                        }
                    }).addOnFailureListener(new OnFailureListener() {
                        @Override
                        public void onFailure(@NonNull Exception exception) {
                        }
                    });
                } catch (IOException e) {
                    e.printStackTrace();
                }

            }

            holder.textView_cardView_description.setText(localItemContent.getItems().get(position).getDescription());
            holder.textView_cardView_distance.setText(Double.toString(localItemContent.getItem(position).getDistanceTo(parentActivity.currentLocationManager.mLastKnownLocation) / 1000.0) + " Km");

            UserContent.User tempUser = parentActivity.firebaseManager.userContent.getUserById(localItemContent.getItem(position).getUserId());
            if(tempUser == null) { holder.textView_cardView_user.setText("Anonymous"); }
            else { holder.textView_cardView_user.setText(tempUser.getNickName()); }

            holder.position = position;
            holder.mView.setOnClickListener(new View.OnClickListener() {

                // On click on item --> Start the detail in the same screen or start the detail activity
                @Override
                public void onClick(View v) {

                    int position = holder.position;
                    parentActivity.launchDetailItemFragmentById(localItemContent.getItems().get(position).getId());

                }

            });

        }

        @Override
        public int getItemCount() {
            return localItemContent.getItems().size();
        }

    }

}
