package mnk.replace.fragments;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapView;
import com.google.android.gms.maps.MapsInitializer;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

import java.util.List;

import mnk.replace.activities.NavigationDrawerActivity;
import mnk.replace.R;
import mnk.replace.model.ItemContent;

/**
 * A simple {@link Fragment} subclass.
 */
public class ElementsMapFragment extends Fragment {

    NavigationDrawerActivity parentActivity;

    // UI
    MapView mMapView;
    public GoogleMap googleMap;
    public static final int DEFAULT_ZOOM = 15;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        parentActivity = ((NavigationDrawerActivity)getActivity());
        View rootView = inflater.inflate(R.layout.fragment_elements_map, container, false);
        mMapView = (MapView) rootView.findViewById(R.id.mapView);
        mMapView.onCreate(savedInstanceState);
        mMapView.onResume(); // needed to get the map to display immediately
        try {
            MapsInitializer.initialize(getActivity().getApplicationContext());
        } catch (Exception e) {
            e.printStackTrace();
        }
        mMapView.getMapAsync(new OnMapReadyCallback() {
            @Override
            public void onMapReady(GoogleMap mMap) {
                googleMap = mMap;
                updateLocationUI();
                updateMapMarkers();
            }
        });

        // ACTION BAR BUTTONS
        if(parentActivity.filterItem != null) {
            parentActivity.actionBarShowFilterItem(true);
            parentActivity.actionBarShowViewItemsSelector(true, false);
        }

        return rootView;
    }

    /**
     * Updates the map's UI settings based on whether the user has granted location permission.
     */
    public void updateLocationUI() {
        if (googleMap == null) {
            return;
        }
        try {
            if (parentActivity.currentLocationManager.mLocationPermissionGranted) {
                googleMap.setMyLocationEnabled(true);
                googleMap.getUiSettings().setMyLocationButtonEnabled(true);
                centerCamera();
            }
        } catch (SecurityException e)  {
            Log.e("Exception: %s", e.getMessage());
        }
    }

    public void centerCamera() {
        if (parentActivity.currentLocationManager.mLastKnownLocation != null) {
            // Set the map's camera position to the current location of the device.
            googleMap.moveCamera(CameraUpdateFactory.newLatLngZoom(
                    new LatLng(parentActivity.currentLocationManager.mLastKnownLocation.getLatitude(),
                            parentActivity.currentLocationManager.mLastKnownLocation.getLongitude()), DEFAULT_ZOOM));
        }
    }

    // MAPVIEW
    public void updateMapMarkers() {
        if(googleMap == null) { return; }
        List<ItemContent.Item> filteredItems = parentActivity.firebaseManager.itemContent.getFilteredItems(parentActivity.currentLocationManager.mLastKnownLocation, parentActivity.filterFragment.filterDistance, parentActivity.filterFragment.filterTags);
        for(int x = 0; x < filteredItems.size(); x++) {
            googleMap.setOnMarkerClickListener(new GoogleMap.OnMarkerClickListener() {
                @Override
                public boolean onMarkerClick(Marker m) {
                    parentActivity.launchDetailItemFragmentById(m.getSnippet());
                    return true;
                }
            });
            googleMap.addMarker(new MarkerOptions()
                    .position(new LatLng(filteredItems.get(x).getLatitude(), filteredItems.get(x).getLongitude()))
                    .title(filteredItems.get(x).getDescription())
                    .snippet(filteredItems.get(x).getId()));
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        mMapView.onResume();
    }

    @Override
    public void onPause() {
        super.onPause();
        mMapView.onPause();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        mMapView.onDestroy();
    }

    @Override
    public void onLowMemory() {
        super.onLowMemory();
        mMapView.onLowMemory();
    }

}