package mnk.replace.manager;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.FragmentTransaction;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.PackageManager;
import android.location.Location;
import android.support.annotation.NonNull;
import android.support.v4.content.ContextCompat;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationResult;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;

import mnk.replace.R;
import mnk.replace.activities.NavigationDrawerActivity;

public class CurrentLocationManager {

    public FusedLocationProviderClient mFusedLocationProviderClient;
    public LocationRequest mLocationRequest;
    public Location mLastKnownLocation;
    public final LatLng mDefaultLocation = new LatLng(-33.8523341, 151.2106085);
    private static final int PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION = 1;
    public boolean mLocationPermissionGranted;
    public NavigationDrawerActivity parentActivity;

    public CurrentLocationManager(NavigationDrawerActivity inActivity) {

        parentActivity = inActivity;
        mFusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(parentActivity);
        getLocationPermission();

    }

    // LOCATION
    /*
     * Prompts the user for permission to use the device location.
     */
    public void getLocationPermission() {
        /*
         * Request location permission, so that we can get the location of the
         * device. The result of the permission request is handled by a callback,
         * onRequestPermissionsResult.
         */
        if (ContextCompat.checkSelfPermission(parentActivity.getBaseContext(),
                android.Manifest.permission.ACCESS_FINE_LOCATION)
                == PackageManager.PERMISSION_GRANTED) {
            mLocationPermissionGranted = true;
            getDeviceLocation();
        }
        else {
            parentActivity.showGPSPermissionDialog();
        }
    }

    public void requestPermissions() {
        parentActivity.requestPermissions(new String[]{android.Manifest.permission.ACCESS_FINE_LOCATION}, PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION);
    }

    /**
     * Handles the result of the request for location permissions.
     */
    public void onRequestPermissionsResult(int requestCode,
                                           @NonNull String permissions[],
                                           @NonNull int[] grantResults) {

        mLocationPermissionGranted = false;
        switch (requestCode) {
            case PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION: {
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    mLocationPermissionGranted = true;
                    getDeviceLocation();
                }
                else {
                    getLocationPermission();
                }
            }
        }
    }

    /**
     * Gets the current location of the device, and positions the map's camera.
     */
    public void getDeviceLocation() {
        /*
         * Get the best and most recent location of the device, which may be null in rare
         * cases when a location is not available.
         */
        try {
            if (mLocationPermissionGranted) {
                Task<Location> locationResult = mFusedLocationProviderClient.getLastLocation();
                locationResult.addOnCompleteListener(new OnCompleteListener<Location>() {
                    @Override
                    public void onComplete(@NonNull Task<Location> task) {
                        mLastKnownLocation = task.getResult();
                        if (mLastKnownLocation == null) {
                            Toast.makeText(parentActivity.getBaseContext(), "Current location is null.", Toast.LENGTH_LONG).show();
                        }
                        else {
                            // LAUNCH MAP'S FRAGMENT
                            ((NavigationDrawerActivity)(parentActivity)).launchLocationActivity();
                        }
                    }
                });
                mLocationRequest = LocationRequest.create()
                        .setPriority(LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY)
                        .setInterval(1000);     // 10 seconds, in milliseconds
                mFusedLocationProviderClient.requestLocationUpdates(mLocationRequest, locationCallback, null /* Looper */);
            }

        }
        catch (SecurityException e)  {
            Log.e("Exception: %s", e.getMessage());
        }
    }

    private LocationCallback locationCallback = new LocationCallback(){
        @Override
        public void onLocationResult(LocationResult locationResult) {
            super.onLocationResult(locationResult);
            if((mLastKnownLocation.getLongitude() != locationResult.getLastLocation().getLongitude()) || (mLastKnownLocation.getLatitude() != locationResult.getLastLocation().getLatitude())) {
                mLastKnownLocation = locationResult.getLastLocation();
            }

        }
    };

}
