package mnk.replace.manager;

import android.content.Intent;
import android.support.annotation.NonNull;
import android.util.Log;
import android.widget.Toast;

import com.google.android.gms.auth.api.Auth;
import com.google.android.gms.auth.api.signin.GoogleSignInAccount;
import com.google.android.gms.auth.api.signin.GoogleSignInOptions;
import com.google.android.gms.auth.api.signin.GoogleSignInResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthCredential;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.auth.GoogleAuthProvider;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import mnk.replace.R;
import mnk.replace.activities.NavigationDrawerActivity;
import mnk.replace.model.ItemContent;
import mnk.replace.model.UserContent;

import static android.content.ContentValues.TAG;

public class FirebaseManager {

    public FirebaseAuth firebaseAuth;
    public FirebaseUser firebaseUser;
    public FirebaseDatabase firebaseDatabase;
    public DatabaseReference firebaseDataReference;
    public NavigationDrawerActivity parentActivity;

    public ItemContent itemContent;
    public UserContent userContent;

    // LOGIN GOOGLE
    public GoogleApiClient mGoogleApiClient;
    private static final int RC_SIGN_IN = 9001;

    public FirebaseManager(NavigationDrawerActivity activity) {

        firebaseAuth = FirebaseAuth.getInstance();
        firebaseUser = firebaseAuth.getCurrentUser();
        firebaseDatabase = FirebaseDatabase.getInstance();
        firebaseDataReference = firebaseDatabase.getReference();
        firebaseDataReference.addValueEventListener(firebaseValueEventListener);
        parentActivity = activity;
        itemContent = new ItemContent();
        userContent = new UserContent();

    }

    public FirebaseAuth getFirebaseAuth() {
        return firebaseAuth;
    }

    public void setFirebaseAuth(FirebaseAuth inFirebaseAuth) { firebaseAuth = inFirebaseAuth; }

    public FirebaseUser getFirebaseUser() {
        return firebaseUser;
    }

    public void setFirebaseUser(FirebaseUser inFirebaseUser) { firebaseUser = inFirebaseUser; }

    public FirebaseDatabase getFirebaseDatabase() {
        return firebaseDatabase;
    }

    public void setFirebaseDatabase(FirebaseDatabase inFirebaseDatabase) { firebaseDatabase = inFirebaseDatabase; }

    public DatabaseReference getFirebaseDataReference() {
        return firebaseDataReference;
    }

    public void setFirebaseDataReference(DatabaseReference inDatabaseReference) { firebaseDataReference = inDatabaseReference; }

    public String getCurrentUserUid() { return firebaseUser.getUid(); }

    public String getCurrentUserEmail() { return firebaseUser.getEmail(); }

    public String getCurrentUserDisplayName() {

        UserContent.User tempUser = userContent.getUserById(getCurrentUserUid());

        if(tempUser == null) {

            String DisplayName = firebaseUser.getDisplayName();
            String Email = getCurrentUserEmail();

            if((DisplayName == null) && (Email.indexOf("@") > -1)) {
                DisplayName = Email.substring(0, Email.indexOf("@"));
            }

            return DisplayName;
        }
        else {
            return tempUser.getNickName();
        }

    }

    public void addCurrentUserToDatabase() {

        DatabaseReference usersRef = firebaseDataReference.child("users/" + firebaseUser.getUid());
        UserContent.User tempUser = new UserContent.User();
        tempUser.setId(getCurrentUserUid());
        tempUser.setEmail(getCurrentUserEmail());
        tempUser.setNickName(getCurrentUserDisplayName());
        usersRef.setValue(tempUser);

    }

    public void updateUserToDatabase(UserContent.User inUser) {

        // Update User Info
        DatabaseReference usersRef = firebaseDataReference.child("users/" + inUser.getId());
        usersRef.setValue(inUser);

    }

    public void updateCurrentUser() { setFirebaseUser(firebaseAuth.getCurrentUser()); }

    // UPDATE DATA
    ValueEventListener firebaseValueEventListener = new ValueEventListener() {
        @Override
        public void onDataChange(DataSnapshot dataSnapshot) {
            // This method is called once with the initial value and again
            // whenever data at this location is updated.
            itemContent.removeItems();
            DataSnapshot headSnapshot = dataSnapshot.child("items");
            for (DataSnapshot itemSnapshot: headSnapshot.getChildren()) {
                itemContent.addItem(itemSnapshot.getValue(ItemContent.Item.class));
            }
            parentActivity.elementsMapFragment.updateMapMarkers();
            if(parentActivity.elementsListFragment.localItemContent != null) { parentActivity.elementsListFragment.refreshItems(); }
            if(parentActivity.detailUserFragment.localItemContent != null) { parentActivity.detailUserFragment.refreshItems(); }
            userContent.removeUsers();
            DataSnapshot headSnapshotUser = dataSnapshot.child("users");
            for (DataSnapshot itemSnapshot: headSnapshotUser.getChildren()) {
                userContent.addUser(itemSnapshot.getValue(UserContent.User.class));
            }
        }
        @Override
        public void onCancelled(DatabaseError databaseError) {
            Toast.makeText(parentActivity.getBaseContext(), "The read failed: " + databaseError.getCode(), Toast.LENGTH_LONG).show();
        }
    };

    public void firebaseAuthWithGoogle(GoogleSignInAccount acct) {
        Log.d(TAG, "firebaseAuthWithGoogle:" + acct.getId());

        AuthCredential credential = GoogleAuthProvider.getCredential(acct.getIdToken(), null);
        firebaseAuth.signInWithCredential(credential)
                .addOnCompleteListener(parentActivity, new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {
                        if (task.isSuccessful()) {
                            // Sign in success, update UI with the signed-in user's information
                            updateCurrentUser();
                            addCurrentUserToDatabase();
                        } else {
                            // If sign in fails, display a message to the user.
                            Toast.makeText(parentActivity.getBaseContext(), "Authentication failed.", Toast.LENGTH_SHORT).show();
                        }
                    }
                });
    }

    // LOGIN GOOGLE
    public void loginGoogle() {
        // Configure Google Sign In
        GoogleSignInOptions gso = new GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
                .requestIdToken(parentActivity.getString(R.string.default_web_client_id))
                .requestEmail()
                .build();
        if(mGoogleApiClient == null) {
            mGoogleApiClient = new GoogleApiClient.Builder(parentActivity.getBaseContext())
                    .enableAutoManage(parentActivity, parentActivity)
                    .addApi(Auth.GOOGLE_SIGN_IN_API, gso)
                    .build();
        }

        Intent signInGoogle = Auth.GoogleSignInApi.getSignInIntent(mGoogleApiClient);
        parentActivity.startActivityForResult(signInGoogle, RC_SIGN_IN);
    }

    public void onActivityResult(int requestCode, int returnCode, Intent data) {
        if(requestCode == RC_SIGN_IN) {
            GoogleSignInResult result = Auth.GoogleSignInApi.getSignInResultFromIntent(data);
            handleSignInResult(result);
        }
    }

    private void handleSignInResult(GoogleSignInResult result) {
        if(result.isSuccess()) {
            GoogleSignInAccount acct = result.getSignInAccount();
            Toast.makeText(parentActivity.getBaseContext(), "Hello " + acct.getDisplayName(), Toast.LENGTH_LONG).show();
            parentActivity.firebaseManager.firebaseAuthWithGoogle(acct);
            parentActivity.updateUIUser();
        }
    }

}
